{****************************************************
 * ZIP-Viewer (Hauptdatei)                          *
 * Delphi-Version 4/5                               *
 *                   Jrgen Hummel, post@hummel1.de *
 ****************************************************}

unit MainZipV;

interface

uses Windows, SysUtils, Classes, Controls, Forms, Dialogs, StdCtrls,
     Grids, Menus;

type TFrmMain = class(TForm)
 ZipGrid: TStringGrid; BtnOK: TButton; BtnView: TButton;
 EdName: TEdit; Label1: TLabel;
 DlgOpen: TOpenDialog;
 MainMenu: TMainMenu;
 mFile: TMenuItem; mReg: TMenuItem; mInfo: TMenuItem;
 mOpen: TMenuItem; N1: TMenuItem; mExit: TMenuItem;
 procedure FormCreate(Sender: TObject);
 procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
 procedure FormResize(Sender: TObject);
 procedure BtnViewClick(Sender: TObject);
 procedure mExitClick(Sender: TObject);
 procedure mOpenClick(Sender: TObject);
 procedure mRegClick(Sender: TObject);
end;

var FrmMain: TFrmMain;

implementation {$R *.DFM}

uses Registry, ZipList;              // in ZipList steht die ZIP-Funktionalitt

procedure TFrmMain.FormCreate(Sender: TObject);
var Reg95: TRegistry;
begin
 Reg95:=TRegistry.Create;
 try                             // Vorsicht, falls die Schlssel nicht da sind
  Reg95.RootKey:=HKEY_CURRENT_USER;
  Reg95.OpenKey('Software\ZipViewer',False);              // privater Schlssel
  Left:=Reg95.ReadInteger('XPos'); Top:=Reg95.ReadInteger('YPos');//priv. Werte
  Width:=Reg95.ReadInteger('XSize'); Height:=Reg95.ReadInteger('YSize');
  Reg95.Free;                                      // Reg95 planmig freigeben
 except Reg95.Free end;                 // auch bei Lesefehlern Reg95 freigeben
 ZipGrid.Cells[0,0]:=' Dateiname';    ZipGrid.Cells[1,0]:=' Datum + Zeit';
 ZipGrid.Cells[2,0]:='Originalgre'; ZipGrid.Cells[3,0]:=' komprimiert';
 ZipGrid.Cells[4,0]:=' Pfad';
 if ParamCount>0 then begin                 // bei Aufruf aus dem Explorer-Men
  EdName.Text:=ParamStr(1);
  BtnViewClick(nil)                        // Archiv einsehen und Gitter fllen
 end;
end;

procedure TFrmMain.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
var Reg95: TRegistry;
begin
 Reg95:=TRegistry.Create;                          // Fensterposition speichern
 Reg95.RootKey:=HKEY_CURRENT_USER;
 try with Reg95 do if OpenKey('Software\ZipViewer',True) then begin
  if not(ValueExists('XPos')) or (ReadInteger('XPos')<>Left) then
   WriteInteger('XPos',Left);
  if not(ValueExists('YPos')) or (ReadInteger('YPos')<>Top) then
   WriteInteger('YPos',Top);
  if not(ValueExists('XSize')) or (ReadInteger('XSize')<>Width) then
   WriteInteger('XSize',Width);
  if not(ValueExists('YSize')) or (ReadInteger('YSize')<>Height) then
   WriteInteger('YSize',Height);
 end; //if OpenKey und with Reg95
 finally
  Reg95.Free;
 end;
end;

procedure TFrmMain.FormResize(Sender: TObject);        // nderung Fenstergre
begin
 EdName.Width:=ClientWidth-220;   BtnView.Left:=ClientWidth-166;
 BtnOK.Left:=ClientWidth-84;      Label1.Top:=ClientHeight-28;
 EdName.Top:=ClientHeight-30;     BtnView.Top:=ClientHeight-29;
 BtnOK.Top:=ClientHeight-29;      ZipGrid.Width:=ClientWidth-15;
 ZipGrid.Height:=ClientHeight-40; ZipGrid.ColWidths[4]:=ClientWidth-384;
end;

procedure TFrmMain.mExitClick(Sender: TObject); begin Close end;

procedure TFrmMain.BtnViewClick(Sender: TObject);
var DirList: TZipList; i: Integer;
begin
 Caption:='ZipViewer'; ZipGrid.Cells[0,0]:= ' Dateiname';
 if FillZipList(EdName.Text,DirList)=ZIP_NO_ERR then begin //fehlerfrei gelesen
  ZipGrid.RowCount:=Length(DirList)+1;
  for i:=0 to High(DirList) do begin                 // Gitter satzweise fllen
   ZipGrid.Cells[0,i+1]:=DirList[i].FileName;
   ZipGrid.Cells[1,i+1]:=FormatDateTime(' dd.mm.yy hh:nn',
                                        FileDateToDateTime(DirList[i].DateTime));
   ZipGrid.Cells[2,i+1]:=FormatFloat('  #00000000',DirList[i].UnCompresSize);
   ZipGrid.Cells[3,i+1]:=FormatFloat('  #00000000',DirList[i].CompresSize);
   ZipGrid.Cells[4,i+1]:=DirList[i].PathName;
  end;
  ZipGrid.Cells[0,0]:= ' Dateinamen ('+IntToStr(Length(DirList))+')';
  Caption:='ZipViewer  [ '+EdName.Text+' ]';
 end else begin                                   // nur eine leere Gitterzeile
  ZipGrid.RowCount:=2;
  for i:=0 to 4 do ZipGrid.Cells[i,1]:='';
 end;
 DirList:=nil;                                      // bergabe-Array freigeben
end;

procedure TFrmMain.mOpenClick(Sender: TObject);
begin
 if DlgOpen.Execute then begin
  EdName.Text:=DlgOpen.Filename;
  BtnViewClick(Sender);
 end;
end;

procedure TFrmMain.mRegClick(Sender: TObject);     // Registrieren des Typs ZIP
var Reg95: TRegistry; sTyp, sTx: String;
begin
 {In dieser Prozedur wird absichtlich keine "with"-Konstruktion verwendet, um }
 {den Gebrauch der Registry-Methoden besonders bersichtlich zu zeigen.       }
 Reg95:=TRegistry.Create;
 Reg95.RootKey:=HKEY_CLASSES_ROOT;
 if Reg95.OpenKey('.zip',True) then begin            {.zip ffnen oder anlegen}
  sTyp:=Reg95.ReadString('');  {evt. vorhandenen Standard-Alias von .zip lesen}
  if Length(sTyp)=0 then begin      {es ist noch kein Standardwert eingetragen}
   sTyp:='ZipFile';     {neue angelegten Schlssel '.zip' als ZipFile erklren}
   Reg95.WriteString('','ZipFile'); {Standardwert fr Schlssel .zip speichern}
  end;
  Reg95.CloseKey;                           {Schlssel '.zip' wieder freigeben}
  sTx:='\'+sTyp+'\shell\view\command';
  if Reg95.OpenKey(sTx,True) then begin  {\ZipFile\Shell\view\command angelegt}
   sTx:=Reg95.ReadString('');        {eventuell vorhandenen Standardwert lesen}
   if Length(sTx)>0 then        {Standardwert war schon da, deshalb nachfragen}
    if mrNo=MessageDlg('Der Typ ".zip" ist bereits mit einer anderen Anwendung'
     +#10+sTx+' verknpft.'#10'Wollen Sie jetzt mit ZipView verknpfen?',
     mtConfirmation,[mbYes, mbNo],0) then begin Reg95.Free; Exit; end;
   Reg95.WriteString('',Application.ExeName+' "%1"');   {Std.-Wert fr command}
   {Der Parameter '%1' wird wegen der langen Dateinamen in " " gesetzt.       }
   Reg95.CloseKey;
   Reg95.OpenKey('\'+sTyp+'\shell\view',False);          {zurck zum Verb view}
   Reg95.WriteString('','&Betrachten'); {Standardwert fr Kontextmen zuordnen}
   Reg95.CloseKey;
   Reg95.OpenKey('\'+sTyp+'\DefaultIcon',True);   {Schlssel fr Standard-Icon}
   if Length(Reg95.ReadString(''))=0 then          {prfen, ob schon vorhanden}
    Reg95.WriteString('',Application.ExeName+',0');       {eigenes Icon setzen}
   Reg95.CloseKey;
   ShowMessage('Der Dateityp ".zip" wurde erfolgreich registriert.');
  end;
 end; {.zip vorhanden/angelegt}
 Reg95.Free;
end;

end.
