{****************************************************
 * Auslesen von Dateiinformationen aus ZIP-Archiven *
 * Delphi-Version 4/5                               *
 *                   Jrgen Hummel, post@hummel1.de *
 ****************************************************}

unit ZipList;

interface

uses classes;

type DirRecord = packed record  // Ergebnis-Datensatz
  FileName       : String;      // Dateiname im Archiv
  PathName       : String;      // Pfadname
  DateTime       : Integer;     // (4) Datum und Uhrzeit
  CompresSize    : Cardinal;    // (4) gepackte Gre
  UncompresSize  : Cardinal;    // (4) ungepackte Gre
end;

type TZipList = array of DirRecord;   // Rckgabe-Array

const ZIP_NO_ERR    = 0;   // ZIP-Analyse erfolgreich
      ZIP_NOT_FOUND = 1;   // ZIP-Datei nicht gefunden
      ZIP_NO_ZIP    = 2;   // Datei ist kein ZIP-Archiv
      ZIP_INVALID   = 3;   // ZIP-Archiv mit fehlerhaftem Format

function FillZipList(ZipName: String; var ZipList: TZipList): Integer;

implementation

uses Windows, SysUtils;

type ZipDirEntry = packed record  // Kennung 'PK12' ( Hex = 02014B50 )
  VersionMade    : Word;      // (2) Version beim Packen
  VersionNeed    : Word;      // (2) Version zum Entpacken
  Flag           : Word;      // (2)
  CompresMethod  : Word;      // (2) Kompressionsmethode
  DateTime       : Integer;   // (4) Datum und Uhrzeit
  CRC32          : Integer;   // (4) CRC32 Prfsumme
  CompresSize    : Cardinal;  // (4) gepackte Gre
  UncompresSize  : Cardinal;  // (4) ungepackte Gre
  FileNameLen    : Word;      // (2) Dateinamenslnge
  ExtraFldLen    : Word;      // (2) ExtraFeldLnge
  FileCommentLen : Word;      // (2) Dateikommentarlnge
  StartOnDisk    : Word;      // (2) Diskettennummer
  IntFileAttrib  : Word;      // (2) internes Dateiattribut
  ExtFileAttrib  : Integer;   // (4) externes Dateiattribut
  RelOffLocalHdr : Cardinal;  // (4) Offset des lokalen DateiHeaders
end;

type ZipEndRecord = packed record // Kennung 'PK56' ( Hex = 06054B50 )
  DiskNo         : Word;      // (2) Disketten-Nummer
  CentralDiskNo  : Word;      // (2) Disketten-Nummer des Zentralverzeichnisses
  CentralEntries : Word;      // (2) Anzahl der Eintrge auf dieser Diskette
  TotalEntries   : Word;      // (2) Anzahl der Eintrge im Zentralverzeichnis
  CentralSize    : Integer;   // (4) Gre des Zentralverzeichnisses
  CentralOffSet  : Cardinal;  // (4) Offset des Zentralverzeichnisses
  ZipCommentLen  : Word;      // (2) Archivkommentarlnge
end;

const LocalFileHeaderSig   = $04034B50;   // 'PK34'
      CentralFileHeaderSig = $02014B50;   // 'PK12'
      EndCentralDirSig     = $06054B50;   // 'PK56'
      ExtLocalSig          = $08074B50;   // 'PK78'

function FillZipList(ZipName: String; var ZipList: TZipList): Integer;
var ZipStream: TFileStream;
    ZipRecord: ZipDirEntry;  //Datenblock wie im CentralFileHeaderDir im Archiv
    EndRecord: ZipEndRecord; //Datenblock nach dem EndCentralDirSig des Archivs
    cBuf: array[0..255]of Char;
    nPos: LongInt; nSign, idx: Integer; nFlag: Byte;
    bEndFound: Boolean;
begin
 SetLength(ZipList,0);                                   // leere Ergebnisliste
 FillZipList:=ZIP_NOT_FOUND;        // Rckgabewert bei Abbruch an diser Stelle
 if not FileExists(ZipName) then Exit;
 ZipStream:=TFileStream.Create(ZipName,fmOpenRead or fmShareDenyNone);
 FillZipList:=ZIP_NO_ZIP;                          // verbesserter Rckgabewert
 try
  if ZipStream.Size>22 then begin      // Zip-End-Record ist mind. 22 Byte gro
   nPos:=ZipStream.Seek(-20,soFromEnd);
   bEndFound:=false;
   while (not bEndFound) and (nPos>0) and (nPos+1024>ZipStream.Size) do begin
    nPos:=ZipStream.Seek(-2,soFromCurrent);      // rckwrts in 1-er Schritten
    ZipStream.Read(nFlag,1);
    if nFlag=Ord('P') then begin              // EndCentralDirSig 'PK56' suchen
     nPos:=ZipStream.Seek(-1,soFromCurrent);  // Zurck aufs 'P'
     ZipStream.Read(nSign,4);
     if nSign=EndCentralDirSig then bEndFound:=true          // 'PK56' gefunden
     else ZipStream.Seek(-4,soFromCurrent);          // vor falsches 'P' zurck
    end;
   end;
   if  bEndFound then begin                  // EndCentralDirSig wurde gefunden
    FillZipList:=ZIP_INVALID;                        // Rckgabewert verbessern
    ZipStream.Read(EndRecord,SizeOf(EndRecord));
    ZipStream.Seek(EndRecord.CentralOffSet,soFromBeginning);//CentralFileHeader
    idx:=0;                             // Zhler fr die gefundenen Dateikpfe
    repeat
     ZipStream.Read(nSign,4);              // CentralFileHeaderSig 'PK12' lesen
     if nSign=CentralFileHeaderSig then begin
      ZipStream.Read(ZipRecord,SizeOf(ZipRecord));          // fixen Teil lesen
      ZipStream.Read(cBuf,ZipRecord.FileNameLen);           // Dateinamen lesen
      cBuf[ZipRecord.FileNameLen]:=#0;           // Dateinamen null-terminieren
      OemToChar(cBuf,cBuf);                     // Namen nach ANSI konvertieren
      while StrScan(cBuf,'/')<>nil do StrScan(cBuf,'/')^:='\';    // '/' in '\'
      ZipStream.Seek(ZipRecord.FileCommentLen,soFromCurrent); //Komm. bergehen
      SetLength(ZipList,idx+1);                 // neuen Ergebnisrecord anlegen
      ZipList[idx].DateTime:=ZipRecord.DateTime;       // Ergebnisrecord fllen
      ZipList[idx].CompresSize:=ZipRecord.CompresSize;
      ZipList[idx].UncompresSize:=ZipRecord.UncompresSize;
      ZipList[idx].FileName:=ExtractFileName(cBuf);
      ZipList[idx].PathName:=ExtractFilePath(cBuf);
      Inc(idx);
     end;
    until nSign<>CentralFileHeaderSig;             // CentralFileHeader zu Ende
    if ZipStream.Position=nPos+4 then FillZipList:=ZIP_NO_ERR;//'PK56' erreicht
   end;  // EndCentralDirSig wurde gefunden
  end;  // >22 Byte Lnge
 finally
  ZipStream.Free;
 end;
end;

end.
